@@ -1601,6 +1601,10 @@ cal_backend_wcap_is_read_only (ECalBackendSync *backend, EDataCal *cal, gboolean
 		return GNOME_Evolution_Calendar_Success;
 	}
 
+	if (!priv->so_cnc || !priv->props) {
+		return GNOME_Evolution_Calendar_OtherError;
+	}
+
 	hasWrite = sunone_util_has_permissions (priv->props, sunone_connection_get_user (priv->so_cnc),
                        SUNONE_ACE_CONTEXT_CALENDAR_COMPONENTS,
                        SUNONE_ACE_PERMISSION_WRITE);
@@ -1708,28 +1712,37 @@ go_online (CalBackendWCAP *wcap)
 	char *key;
 	const char *uristr;
 
+	g_mutex_lock (priv->open_lock);
+
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-	if (!priv->uri)
-		priv->uri = g_strdup (uristr);
-
-	if (!priv->calid)
-		priv->calid = sunone_util_get_calid_from_uri (uristr);
-
-	if (!priv->cache_name)
-		priv->cache_name = get_cache_name (wcap, TRUE);
-
-	account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
+	account = sunone_component_get_account_from_uri (global_sunone_component, uristr);
 	if (!account) {
-		cal_backend_wcap_clean_wcap (wcap, TRUE);
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 	cnc = sunone_account_get_connection (account);
 	if (!cnc) {
-		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
+
+	if (priv->calid)
+		g_free (priv->calid);
+	priv->calid = sunone_util_get_calid_from_uri (uristr);
+	if (!priv->calid) {
+		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
+		return GNOME_Evolution_Calendar_OtherError;
+	}
+	if (priv->uri)
+		g_free (priv->uri);
+	priv->uri = g_strdup (uristr);
+	if (priv->cache_name)
+		g_free (priv->cache_name);
+	priv->cache_name = get_cache_name (wcap, TRUE);
 	priv->so_cnc = g_object_ref (cnc);
+	read_cache (wcap);
 
 	/* load timezones */
 	error_code = sunone_connection_get_all_timezones (priv->so_cnc, &icalcomp);
@@ -1737,6 +1750,7 @@ go_online (CalBackendWCAP *wcap)
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
@@ -1751,6 +1765,7 @@ go_online (CalBackendWCAP *wcap)
 	if (!priv->props) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 	
@@ -1759,6 +1774,7 @@ go_online (CalBackendWCAP *wcap)
 	if (!prefs) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
@@ -1791,6 +1807,7 @@ go_online (CalBackendWCAP *wcap)
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
 		g_object_unref (account);
+		g_mutex_unlock (priv->open_lock);
 
 		if (error_code == SUNONE_ERROR_ACCESS_DENIED)
 			return GNOME_Evolution_Calendar_PermissionDenied;
@@ -1818,6 +1835,8 @@ go_online (CalBackendWCAP *wcap)
 
 	cal_backend_wcap_write_cache (wcap);
 
+	g_mutex_unlock (priv->open_lock);
+
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1827,62 +1846,56 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 {
 	CalBackendWCAP *wcap = (CalBackendWCAP *) backend;
 	CalBackendWCAPPrivate *priv = wcap->priv;
-	char *type;
-	const char *uristr;
+	icalcomponent_kind kind;
+	GThread *thread = NULL;
+	GError *error = NULL;
 	int ret;
-	
+
 	d(printf("cal_backend_wcap_open(%p, %d)\n", backend, only_if_exists));
 
 	/* check whether already open */
 	if (priv->uri)
 		return GNOME_Evolution_Calendar_Success;
 
-	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-
-	g_mutex_lock (priv->open_lock);
-
-	priv->uri = g_strdup (uristr);
-	
-	priv->calid = sunone_util_get_calid_from_uri (uristr);
-	if (!priv->calid) {
-		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_mutex_unlock (priv->open_lock);
-		return GNOME_Evolution_Calendar_NoSuchCal;
-	}
-
-	type = sunone_util_get_parameter_from_uri (uristr, "type");
-	if (type && !strcmp (type, "tasks"))
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND(wcap));
+	if (kind == ICAL_VTODO_COMPONENT)
 		priv->type = TYPE_TODO;
-	else if (type && !strcmp (type, "calendar"))
+	else if (kind == ICAL_VEVENT_COMPONENT)
 		 priv->type = TYPE_EVENT;
-	g_free (type);
-
-	priv->cache_name = get_cache_name (wcap, TRUE);
 
 	if (priv->mode == CAL_MODE_LOCAL) {
 		ESource *source;
+		const char *uristr;
 		const char *display_contents = NULL;
-			
+
+		uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
+		priv->calid = sunone_util_get_calid_from_uri (uristr);
+		if (!priv->calid)
+			return GNOME_Evolution_Calendar_NoSuchCal;
+		priv->uri = g_strdup (uristr);
+		priv->cache_name = get_cache_name (wcap, TRUE);
+
 		source = e_cal_backend_get_source (E_CAL_BACKEND (backend));
 		display_contents = e_source_get_property (source, "offline_sync");
-		
 		if (!display_contents || !g_str_equal (display_contents, "1")) {
 			cal_backend_wcap_clean_wcap (wcap, TRUE);
-			g_mutex_unlock (priv->open_lock);
 			return GNOME_Evolution_Calendar_RepositoryOffline;
 		}
 
 		read_cache (wcap);
 
-		g_mutex_unlock (priv->open_lock);
 		return GNOME_Evolution_Calendar_Success;
 	}
 
-	read_cache (wcap);
-	ret = go_online (wcap);
-	g_mutex_unlock (priv->open_lock);
+	thread = g_thread_create ((GThreadFunc)go_online, wcap, FALSE, &error);
+	if (!thread) {
+		g_warning (G_STRLOC ": %s", error->message);
+		g_error_free (error);
 
-	return ret;
+		return GNOME_Evolution_Calendar_OtherError;
+	}
+
+	return GNOME_Evolution_Calendar_Success;
 }
 
 static ECalBackendSyncStatus
@@ -2044,7 +2057,6 @@ cal_backend_wcap_get_object_list (ECalBackendSync *backend, EDataCal *cal,
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	struct search_data match_data;
 	const char *uristr;
-	char *type;
 
 	g_return_val_if_fail (sexp != NULL, GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (objects != NULL, GNOME_Evolution_Calendar_InvalidObject);
@@ -2064,14 +2076,11 @@ cal_backend_wcap_get_object_list (ECalBackendSync *backend, EDataCal *cal,
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
 
-	type = sunone_util_get_parameter_from_uri (uristr, "type");
 	g_hash_table_foreach (priv->objects, match_object, &match_data);
 
 	g_object_unref (match_data.sexp);
 	*objects = match_data.matches;
 
-	g_free (type);
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -2237,8 +2246,9 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	CalBackendWCAP *wcap = CAL_BACKEND_WCAP (backend);
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	SunOneAccount *account;
+	icalcomponent_kind kind;
 	const char *uristr;
-	char *dirname, *filename, *type;
+	char *dirname, *filename;
 	EXmlHash *xmlhash;
 	struct ChangeData data;
 	struct cbw_data cbwdata;
@@ -2251,18 +2261,18 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
 
-	type = sunone_util_get_parameter_from_uri (uristr, "type");
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (backend));
 
 	/* open the changes file */
 	account = sunone_component_get_account_from_uri (global_sunone_component, priv->uri);
 	if (!account)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	
-	if (type && !strcmp(type, "calendar"))
+	if (kind ==  E_CAL_COMPONENT_EVENT)
 		dirname = g_strdup_printf ("%s/sunone/%s@%s/Calendar", evolution_dir,
 					   sunone_account_get_user (account),
 					   sunone_account_get_server (account));
-	else if (type && !strcmp(type, "tasks"))
+	else if (kind == E_CAL_COMPONENT_TODO)
 		dirname = g_strdup_printf ("%s/sunone/%s@%s/Tasks", evolution_dir,
 					   sunone_account_get_user (account),
 					   sunone_account_get_server (account));
@@ -2290,7 +2300,7 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 	data.ehash = xmlhash;
 	data.adds = NULL;
 	data.modifies = NULL;
-	if (type && (!strcmp(type, "calendar") || !strcmp(type, "tasks")))
+	if (kind == E_CAL_COMPONENT_EVENT || kind == E_CAL_COMPONENT_TODO)
 		g_hash_table_foreach (priv->objects, check_change_type, &data);
 	else
 		return GNOME_Evolution_Calendar_OtherError;
@@ -2315,8 +2325,6 @@ cal_backend_wcap_get_changes (ECalBackendSync *backend, EDataCal *cal, const cha
 
 	e_xmlhash_destroy (xmlhash);
 
-	g_free (type);
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
